<?php
namespace Teknisa\COZ\Services;

use Teknisa\Libs\Util\Environment;
use Teknisa\COZ\Services\Pla10500Repository;
use Teknisa\COZ\Services\Pla10500RemoteRepository;

class Pla10500Service{

    protected $environment;
    protected $connection;
    protected $pla10500repository;
    protected $pla10500remoteRepository;

    public function __construct(Environment $environment,  $connection
        , Pla10500Repository $pla10500repository, Pla10500RemoteRepository $pla10500remoteRepository){
        $this->environment = $environment;
        $this->connection = $connection;
        $this->pla10500repository = $pla10500repository;
        $this->pla10500remoteRepository = $pla10500remoteRepository;
    }

    public function geraPlanejamento(&$row){

        // $row['LISTAGRUPFILI'] = $this->retornaGrupfili((array)$row);
        if($row['CDGRUPFILI'] == '') {
            $row['CDGRUPFILI'] = 'T';
        }
        $row['LISTAGRUPFILI'] = $row['CDGRUPFILI'];
        $row['LISTAFILIAGREGA'] = $this->retornaFiliaisagregadas((array)$row);
        $row['LISTAGRUPSERV'] = $this->retornaGrupserv((array)$row);

        $row['SERVICOFILIAL'] = $this->retornaServicoporfiliais((array)$row);
        $row['LISTASERVFILI'] = $this->concatArray($row['SERVICOFILIAL'], 'SERVICO');

        $row['LISTATIPOPRAT'] = $this->retornaTipoprat((array)$row);
        $row['LISTASETORCOZ'] = $this->retornaSetorcoz((array)$row);

        $row['DTCARDAFIN_FILTRO'] = $row['DTCARDAFIN'];
        $row['DTCARDAFIN'] = $this->retornaDataFinal((array)$row);
        
        $row['IDOBRIGINFOQTD'] = $this->retornaIdObrigaInfo($row);
        $nrplanprod = $this->pla10500repository->geraPlanProducao($row, 'G');
        $this->pla10500repository->deletaConsplan();

        return $nrplanprod;
    }
    
    public function retornaIdObrigaInfo($row){
        	$sql = "SELECT IDOBRIGINFOQTD FROM PARAMPLANEJA WHERE CDFILIAL =:CDFILIAL";
 	              
     	    $params = array(
     	        "CDFILIAL" => $row['CDCENTRAL']
     		);
 			$result = $this->connection->fetchColumn($sql, $params);
 			return $result;
    }

    public function cancelaPlanejamento($row){
        $fieldsToFilter = array("NRORG", "NRPLANPROD");
        $paramObjectArray = $this->buildFilterArrayFromRow((array)$row, $fieldsToFilter);
        $idconcluido = $this->pla10500remoteRepository->fetchCancelaPlanProducao($paramObjectArray);
        if(empty($idconcluido)){
            if(!empty($row["CDFILIAL"]) && !empty($row["NRPLANPROD"])
                && !empty($row["DTPRODUCAO"]) && !empty($row["DTCARDAPIO"])){

                $row['DTCARDAFIN_FILTRO'] = $row['DTCARDAFIN'];
                $row['DTCARDAFIN'] = $this->retornaDataFinal((array)$row);

                $this->pla10500repository->geraPlanProducao($row, 'C');
            } else{
                throw new \Exception('Não há Planejamento da Produção para ser cancelado!');
            }
        } else{
            throw new \Exception('Não foi possível cancelar o Planejamento da Produção. Há produtos que estão concluídos!');
        }
    }


    public function retornaPlanejamento(&$row){
        $fieldsToFilter = array("NRORG", "CDCENTRAL", "NRPLANPROD");
        $paramObjectArray = $this->buildFilterArrayFromRow((array)$row, $fieldsToFilter);
        $planproducao = $this->pla10500remoteRepository->fetchPlanproducao($paramObjectArray);
        if(empty($planproducao)){
            return $planproducao;
        }else{
            return $planproducao[0];
        }
    }

    public function retornaValoresProdutoNovaReceita(&$row){
        $fieldsToFilter = array("CDFILIAL", "CDPRODUTO", "QTPRODRECEPRAT", "DTCONSUMO");
        $paramObjectArray = $this->buildFilterArrayFromRow((array)$row, $fieldsToFilter);
        $valoresproduto = $this->pla10500remoteRepository->fetchValoresProdutoNovaReceita($paramObjectArray);
        return $valoresproduto;
    }

    public function retornaTipopratoNovaReceita(&$row){
        $fieldsToFilter = array("NRPLANPROD", "CDPRODACABADO", "CDFILIAGREG", "CDSERVFILIAGREG", "DTCONSUMO");
        $paramObjectArray = $this->buildFilterArrayFromRow((array)$row, $fieldsToFilter);
        $cdtipoprat = $this->pla10500remoteRepository->fetchTipopratoNovaReceita($paramObjectArray);
        return $cdtipoprat;
    }

    public function retornaGrupfili($row){
        if($row['T_NMGRUPFILI'] === 'S'){
            $row['NMGRUPFILI'] = $this->pla10500remoteRepository->fetchGrupfili();
        }
        return $this->concatArray($row['NMGRUPFILI']);
    }

    public function retornaFiliaisagregadas($row){
        if($row['T_NMFILIAGREGA'] === 'S'){
            $row['CDGRUPFILI'] = $row['LISTAGRUPFILI'];

            //valores que o filtro de filiais agregadar precisa
            $fieldsToFilter = array("NRORG", "CDGRUPFILI", "CDCENTRAL");

            $paramObjectArray = $this->buildFilterArrayFromRow($row, $fieldsToFilter);

            $row['NMFILIAGREGA'] = $this->pla10500remoteRepository->fetchFiliaisagregadas($paramObjectArray);
        }
        return $this->concatArray($row['NMFILIAGREGA']);
    }

    public function retornaGrupserv($row){
        if($row['T_NMGRUPSERV'] === 'S'){
            $row['NMGRUPSERV'] = $this->pla10500remoteRepository->fetchGrupserv();
        }
        return $this->concatArray($row['NMGRUPSERV']);
    }

    public function retornaServicoporfiliais($row){
        if($row['T_NMSERVFILI'] === 'S'){
            $row['CDGRUPFILI'] = $row['LISTAGRUPFILI'];
            $row['CDGRUPSERV'] = $row['LISTAGRUPSERV'];
            $row['CDFILIAGREGA'] = $row['LISTAFILIAGREGA'];

            //valores que o filtro de serviços precisa
            $fieldsToFilter = array("NRORG", "CDGRUPFILI", "CDCENTRAL", "CDGRUPSERV", "CDFILIAGREGA");

            $paramObjectArray = $this->buildFilterArrayFromRow($row, $fieldsToFilter);
            $row['SERVICOFILIAL'] = $this->pla10500remoteRepository->fetchServicos($paramObjectArray);
        }
        return $row['SERVICOFILIAL'];
    }

    public function retornaTipoprat($row){
        if($row['T_NMTIPOPRAT'] === 'S'){
            $row['NMTIPOPRAT'] = $this->pla10500remoteRepository->fetchTipoPrato();
        }
        return $this->concatArray($row['NMTIPOPRAT']);
    }

    public function retornaSetorcoz($row){
        if($row['T_NMSETORCOZ'] === 'S'){
            $row['NMSETORCOZ'] = $this->pla10500remoteRepository->fetchSetorcoz();
        }
        return $this->concatArray($row['NMSETORCOZ']);
    }

    public function retornaDataFinal($row){
        $fieldsToFilter = array("DTCARDAFIN");
        $paramObjectArray = $this->buildFilterArrayFromRow($row, $fieldsToFilter);
        $dtcardafin = $this->pla10500remoteRepository->fetchDataFinal($paramObjectArray);
        return $dtcardafin[0];
    }

    public function atualizaPlanprodfiliagrega($row){
        return $this->pla10500repository->atualizaPlanprodfiliagrega($row);
    }

    public function atualizaPlanprodacabado($row){
        return $this->pla10500repository->atualizaPlanprodacabado($row);
    }

    public function atualizaPaiProduzirProdacabado($row){
        return $this->pla10500repository->atualizaPaiProduzirProdacabado($row);
    }

    public function atualizaFilhoProduzirProdacabado($row){
        return $this->pla10500repository->atualizaFilhoProduzirProdacabado($row);
    }

    public function atualizaFilhoProdFiliAgrega($row){
        return $this->pla10500repository->atualizaFilhoProdFiliAgrega($row);
    }

    public function retornaFilhosProduzidos($row){
        $fieldsToFilter = array("NRORG", "NRPLANPROD", "CDPRODUTO");
        $paramObjectArray = $this->buildFilterArrayFromRow((array)$row, $fieldsToFilter);
        $produzidos = $this->pla10500remoteRepository->fetchFilhosProduzidos($paramObjectArray);
        return $produzidos;
    }

    public function retornaProdutoJaProduzido($row){
        $fieldsToFilter = array("CDPRODUTO", "CDFILIAGREG", "CDSERVFILIAGREG", "IDCOOKCHILL", "IDTPPRODCHILL", "IDFINALIZACAO", "IDESTADO", "DTCONSUMO");
        $paramObjectArray = $this->buildFilterArrayFromRow((array)$row, $fieldsToFilter);
        $produzido = $this->pla10500remoteRepository->fetchProdutoJaProduzido($paramObjectArray);
        return $produzido;
    }

    public function retornaProdutosProduzidos($row){
        try {
            $fieldsToFilter = array("NRPLANPROD", "CDPRODUTO", "IDCOOKCHILL", "IDTPPRODCHILL", "IDFINALIZACAO", "IDESTADO", "DTCONSUMOINI", "DTCONSUMOFIN");
            $paramObjectArray = $this->buildFilterArrayFromRow((array)$row, $fieldsToFilter);
            $produzidos = $this->pla10500remoteRepository->fetchProdutosProduzidos($paramObjectArray);
            return $produzidos;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function concatArray($array, $key = false){
        if ($key){
            $string = '';
            foreach ($array as $item) {
              $string .= $item[$key].'_';
            }
            return trim($string, '_');
        }
        else return implode($array, "_");
    }

    /*
     * Constroi um array de arrays a partir de uma row, no formato chave=>valor,
     * e um array de strings especificando quais das colunas deverão ser usadas
     */
    public function buildFilterArrayFromRow($row, $filterKeys){

        //pega todos os valores de $row com chaves presentes em $filterKeys
        //o array_flip faz com que os valores de $filterKeys sejam chaves
        $paramValueArray = array_intersect_key($row, array_flip($filterKeys));

        return array_map(function($key, $value){
            return array("columnName"=>$key, "operator"=>"=", "value"=>$value);
        }, array_keys($paramValueArray), $paramValueArray);
    }

    public function proceAlteraQtdeProdacabado($row){
        return $this->pla10500repository->proceAlteraQtdeProdacabado($row);
    }

    public function insereJustPlanProd($justificativa){
        return $this->pla10500repository->insereJustPlanProd($justificativa);
    }

    public function verificaEstoqueProdutosReceita($nrorg, $central, $nrPlanprod, $lista, $data){
        return $this->pla10500remoteRepository->verificaEstoqueProdutosReceita(
            $nrorg, $central, $nrPlanprod, $lista, $data
        );
    }

    public function salvaLogMP(
        $nrorg, $nrPlanprod, $codigo, $origem, $produtos, $prodSaldoInsuf, $operador, $nrlista = null
    ){
        return $this->pla10500remoteRepository->salvaLogMP(
            $nrorg, $nrPlanprod, $codigo, $origem, $produtos, $prodSaldoInsuf, $operador, $nrlista
        );
    }

    public function atualizaLogMP($nrorg, $codigo, $justificativa){
        return $this->pla10500repository->atualizaLogMP(
            $nrorg, $codigo, $justificativa
        );
    }

    public function geraPedidoProducao($row){
        try {
            $nrlogmatprima = $this->pla10500repository->geraPedidoProducao($row);
            return $nrlogmatprima;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function verificaEstoqueProdutosReceitaComReserva($nrorg, $central, $nrPlanprod, $lista, $data){
        return $this->pla10500remoteRepository->verificaEstoqueProdutosReceitaComReserva(
            $nrorg, $central, $nrPlanprod, $lista, $data
        );
    }

    public function proceLiberaProducao($row){
        return $this->pla10500repository->proceLiberaProducao($row);
    }
}